package types

import (
	"database/sql"
	"database/sql/driver"
	"electricity_bill_calc/tools"
	"encoding/json"
	"errors"

	"github.com/jackc/pgx/v5/pgtype"
)

type DateRange struct {
	pgtype.Range[Date]
}

func NewEmptyDateRange() DateRange {
	return DateRange{
		Range: pgtype.Range[Date]{
			Lower:     MinDate(),
			LowerType: pgtype.Unbounded,
			Upper:     MaxDate(),
			UpperType: pgtype.Unbounded,
			Valid:     true,
		},
	}
}

func NewDateRange(lower *Date, upper *Date) DateRange {
	return DateRange{
		Range: pgtype.Range[Date]{
			LowerType: tools.Cond(lower != nil, pgtype.Inclusive, pgtype.Unbounded),
			Lower:     tools.DefaultTo(lower, MinDate()),
			UpperType: tools.Cond(upper != nil, pgtype.Inclusive, pgtype.Unbounded),
			Upper:     tools.DefaultTo(upper, MaxDate()),
			Valid:     true,
		},
	}
}

var _ driver.Valuer = (*DateRange)(nil)

func (dr DateRange) Value() (driver.Value, error) {
	return assembleRange(dr.Range), nil
}

var _ sql.Scanner = (*DateRange)(nil)

func (dr *DateRange) Scan(src interface{}) (err error) {
	switch src := src.(type) {
	case pgtype.Range[Date]:
		dr.Range = src
	case string:
		r, err := destructureToRange[Date](src)
		if err != nil {
			return err
		}
		dr.Range = r
	case []byte:
		r, err := destructureToRange[Date](string(src))
		if err != nil {
			return err
		}
		dr.Range = r
	case nil:
		dr = nil
	default:
		return errors.New("该数据类型不支持解析到日期范围。")
	}
	return
}

var _ json.Marshaler = (*DateRange)(nil)

func (dr DateRange) MarshalJSON() ([]byte, error) {
	return json.Marshal(assembleRange(dr.Range))
}

var _ json.Unmarshaler = (*DateRange)(nil)

func (dr *DateRange) UnmarshalJSON(data []byte) error {
	var str string
	if err := json.Unmarshal(data, &str); err != nil {
		return errors.New("不能解析指定的日期范围值。")
	}
	r, err := destructureToRange[Date](str)
	if err != nil {
		return err
	}
	dr.Range = r
	return nil
}

func (dr *DateRange) SetLower(lower Date, bound ...pgtype.BoundType) {
	bound = append(bound, pgtype.Inclusive)
	dr.Range.Lower = lower
	dr.Range.LowerType = bound[0]
}

func (dr *DateRange) SetLowerUnbounded() {
	dr.Range.Lower = MinDate()
	dr.Range.LowerType = pgtype.Unbounded
}

func (dr *DateRange) SetUpper(upper Date, bound ...pgtype.BoundType) {
	bound = append(bound, pgtype.Inclusive)
	dr.Range.Upper = upper
	dr.Range.UpperType = bound[0]
}

func (dr *DateRange) SetUpperUnbounded() {
	dr.Range.Upper = MaxDate()
	dr.Range.UpperType = pgtype.Unbounded
}
